#include <proto.h>

#include "gtcolormap.h"
#include "gterror.h"

/************************************************************************\
*
* GtColormap class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init colormap.
*
\************************************************************************/
GtColormap *GtMkColormap(GtDisplay *d,GtTag *tgs)
{
	GtColormap *obj;
	
	if(!(obj=new GtColormap()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtColormap object!\n");
		return(NULL);
	}
	if(!obj->Init(d,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init colormap.
*
\************************************************************************/
GtColormap *GtMkColormap(GtDisplay *d,ulong tag,...)
{
	va_list va;
	GtColormap *obj;
	int p;
	
	va_start(va,tag);
	p=GtBeginVaTags();
	obj=GtMkColormap(d,GtCopyVaTags(tag,&va));
	GtEndVaTags(p);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty colormap.
*
\************************************************************************/
GtColormap::GtColormap(void)
{
	dsp=NULL;
	vis=NULL;
	cm=0;
	mode=GTCM_DEFAULT;
	nofreex=FALSE;
}

/************************************************************************\
*
* Destroy colormap.
*
\************************************************************************/
void GtColormap::Destroy(void)
{
	if(cm && (mode!=GTCM_DEFAULT) && !nofreex)
		XFreeColormap(dsp->XDsp(),cm);
	GtBase::Destroy();
}

/************************************************************************\
*
* Setup colormap.
*
\************************************************************************/
sbool GtColormap::Init(GtDisplay *d,GtTag *tgs)
{
	GtTag *t;
	int al=0;
	
	dsp=d;
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Mode:
				mode=t->d.val;
				if(mode>GTCM_ALLOCNONE)
					mode=GTCM_DEFAULT;
				break;
			case GT_Visual:
				vis=(Visual *)t->d.ptr;
				break;
			case GT_XColormap:
				cm=(Colormap)t->d.ptr;
				nofreex=TRUE;
				break;
			case GT_DontFreeX:
				if(t->d.val)
					nofreex=TRUE;
				else
					nofreex=FALSE;
				break;
		}
	}
	if(!dsp)
	{
		GtError(this,GTEC_NODISPLAY,"No GtDisplay!\n");
		return(FALSE);
	}
	if(!vis)
		vis=dsp->XDefaultVisual();
	if(!cm)
	{
		if(mode==GTCM_DEFAULT)
			cm=dsp->XDefaultColormap();
		else
		{
			if(mode==GTCM_ALLOCNONE)
				al=AllocNone;
			cm=XCreateColormap(dsp->XDsp(),dsp->XDefaultRootWindow(),vis,al);
		}
	}
	if(!cm)
	{
		GtError(this,GTEC_NOXRESOURCE,"No X-Colormap!\n");
		return(FALSE);
	}
	return(TRUE);
}

/************************************************************************\
*
* Find display tag and then setup normal.
*
\************************************************************************/
sbool GtColormap::Init(GtTag *tgs)
{
	GtDisplay *d;
	
	d=(GtDisplay *)GtGetTagPtr(tgs,GT_Display,NULL);
	return(this->Init(d,tgs));
}

/************************************************************************\
*
* Set colormap attributes.
*
\************************************************************************/
sbool GtColormap::Set(GtTag *tgs)
{
	tgs=tgs;
	return(TRUE);
}

/************************************************************************\
*
* Get colormap attributes.
*
\************************************************************************/
void GtColormap::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Display:
				*ptr=(long)dsp;
				break;
			case GT_XColormap:
				*ptr=(long)cm;
				break;
			case GT_Mode:
				*ptr=mode;
				break;
			case GT_Visual:
				*ptr=(long)vis;
				break;
			case GT_DontFreeX:
				*ptr=nofreex;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtColormap::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtColormap")==0)
		return(TRUE);
	else
		return(GtBase::IsSubClassOf(cn));
}

/************************************************************************\
*
* Alloc color.
*
\************************************************************************/
sbool GtColormap::AllocC(XColor *rgb)
{
	if(!XAllocColor(dsp->XDsp(),cm,rgb))
		return(FALSE);
	return(TRUE);
}

/************************************************************************\
*
* Alloc color.
*
\************************************************************************/
sbool GtColormap::AllocC(ulong *pix,uword r,uword g,uword b)
{
	XColor xc;
	
	xc.red=r;
	xc.green=g;
	xc.blue=b;
	xc.flags=DoRed|DoGreen|DoBlue;
	if(!XAllocColor(dsp->XDsp(),cm,&xc))
		return(FALSE);
	*pix=xc.pixel;
	return(TRUE);
}

/************************************************************************\
*
* Alloc named color.
*
\************************************************************************/
sbool GtColormap::AllocNamedC(char *name,XColor *rgb)
{
	XColor xc;
	
	if(!XAllocNamedColor(dsp->XDsp(),cm,name,rgb,&xc))
		return(FALSE);
	return(TRUE);
}

/************************************************************************\
*
* Alloc color cells.
*
\************************************************************************/
sbool GtColormap::AllocCells(ulong *cells,int nr,sbool cont)
{
	if(!XAllocColorCells(dsp->XDsp(),cm,(cont ? True : False),NULL,0,cells,nr))
		return(FALSE);
	return(TRUE);
}

/************************************************************************\
*
* Free color.
*
\************************************************************************/
void GtColormap::FreeC(ulong pix)
{
	XFreeColors(dsp->XDsp(),cm,&pix,1,0);
}

/************************************************************************\
*
* Store color.
*
\************************************************************************/
void GtColormap::StoreC(XColor *rgb)
{
	XStoreColor(dsp->XDsp(),cm,rgb);
}

/************************************************************************\
*
* Store color.
*
\************************************************************************/
void GtColormap::StoreC(ulong pix,uword r,uword g,uword b)
{
	XColor xc;
	
	xc.pixel=pix;
	xc.red=r;
	xc.green=g;
	xc.blue=b;
	xc.flags=DoRed|DoGreen|DoBlue;
	XStoreColor(dsp->XDsp(),cm,&xc);
}

/************************************************************************\
*
* Store named color.
*
\************************************************************************/
void GtColormap::StoreNamedC(char *name,ulong pix)
{
	XStoreNamedColor(dsp->XDsp(),cm,name,pix,DoRed|DoGreen|DoBlue);
}

/************************************************************************\
*
* Query color.
*
\************************************************************************/
void GtColormap::QueryC(XColor *rgb)
{
	XQueryColor(dsp->XDsp(),cm,rgb);
}

/************************************************************************\
*
* Find RGB values of named color.
*
\************************************************************************/
sbool GtColormap::FindNamedC(char *name,XColor *rgb,XColor *trgb)
{
	XColor xc;
	
	if(trgb)
	{
		if(!XLookupColor(dsp->XDsp(),cm,name,trgb,rgb))
			return(FALSE);
	}
	else
	{
		if(!XLookupColor(dsp->XDsp(),cm,name,&xc,rgb))
			return(FALSE);
	}
	return(TRUE);
}
